@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho.pm 6422 2007-04-09T02:27:47.388429Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho.pm 6474 2007-04-12T00:05:39.347756Z lestrrat  $
 # 
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -6,17 +6,20 @@
 package Gungho;
 use strict;
 use warnings;
-use base qw(Class::Accessor::Fast);
+use 5.008;
+use base qw(Gungho::Base);
 use Carp qw(croak);
 use Config::Any;
 use Class::Inspector;
+use UNIVERSAL::isa;
 use UNIVERSAL::require;
 
 use Gungho::Log;
+use Gungho::Exception;
 
-__PACKAGE__->mk_accessors($_) for qw(config log provider handler engine hooks);
+__PACKAGE__->mk_accessors($_) for qw(config log provider handler engine is_running hooks features);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub new
 {
@@ -35,13 +38,35 @@ sub setup
     my $self = shift;
 
     $self->hooks({});
+    $self->features({});
 
+    $self->setup_components();
     $self->setup_log();
     $self->setup_provider();
     $self->setup_handler();
     $self->setup_engine();
 
     $self->setup_plugins();
+
+    $self->next::method(@_); # This propagates to 
+}
+
+sub setup_components
+{
+    my $self = shift;
+
+    my $list = $self->config->{components};
+    foreach my $module (@$list) {
+        my $pkg = $self->load_gungho_module($module, 'Component');
+        $pkg->inject_base($self);
+    }
+
+    # XXX - Hack! Class::C3 doesn't like it when we have G::Base
+    # before G::Component based objects in our ISA, so remove
+    if (@$list) {
+        @Gungho::ISA = grep { $_ ne 'Gungho::Base' } @Gungho::ISA;
+        Class::C3::reinitialize();
+    }
 }
 
 sub setup_log
@@ -135,6 +160,12 @@ sub run_hook
     }
 }
 
+sub has_feature
+{
+    my ($self, $name) = @_;
+    return exists $self->features()->{$name};
+}
+
 sub load_config
 {
     my $self = shift;
@@ -173,30 +204,48 @@ sub load_gungho_module
     return $pkg;
 }
 
-sub run { $_[0]->engine->run($_[0]) }
-
-sub has_requests
+sub run
 {
-    my $self = shift;
-    $self->provider->has_requests;
+    $_[0]->is_running(1);
+    $_[0]->engine->run($_[0]);
 }
 
-sub get_requests
+sub dispatch_requests
 {
     my $self = shift;
-    $self->provider->get_requests;
+    $self->provider->dispatch($self);
+}
+
+sub prepare_request
+{
+    my $self = shift;
+    my $req  = shift;
+    $self->run_hook('dispatch.prepare_request', $req);
+
+    return $req;
 }
 
 sub send_request
 {
-    my $self = shift;
-    $self->engine->send_request($_[0]);
+    my ($self, $request) = @_;
+
+    if ($self->has_feature('Throttle')) {
+        if ($self->throttle($request)) {
+            $self->engine->send_request($self, $request);
+        } else {
+            $self->log->debug("Request " . $request->url . " (" . $request->id . ") was throttled")
+                if $self->log->is_debug;
+            Gungho::Exception::RequestThrottled->throw($request);
+        }
+    } else {
+        $self->engine->send_request($self, $request);
+    }
 }
 
 sub handle_response
 {
-    my ($self, $request, $response) = @_;
-    $self->handler->handle_response($self, $request, $response);
+    my ($self) = @_;
+    $self->handler->handle_response(@_);
 }
 
 1;
@@ -253,11 +302,17 @@ or a hashref.
 
 Starts the Gungho process.
 
+=head2 has_feature($name)
+
+Returns true if Gungho supports some feature $name
+
 =head2 setup()
 
 Sets up the Gungho environment, including calling the various setup_*
 methods to configure the provider, engine, handler, etc.
 
+=head2 setup_components()
+
 =head2 setup_engine()
 
 =head2 setup_handler()
@@ -290,6 +345,14 @@ Delegates to provider's get_requests
 
 Delegates to handler's handle_response
 
+=head2 dispatch_requests
+
+Calls provider->dispatch
+
+=head2 prepare_request($req)
+
+Given a request, preps it before sending it to the engine
+
 =head2 send_request
 
 Delegates to engine's send_request
