@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 6421 2007-04-09T02:17:43.124659Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 6470 2007-04-11T23:50:49.863502Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -11,7 +11,7 @@ use POE;
 use POE::Component::Client::Keepalive;
 use POE::Component::Client::HTTP;
 
-__PACKAGE__->mk_accessors($_) for qw(alias);
+__PACKAGE__->mk_accessors($_) for qw(alias loop_alarm loop_delay);
 
 use constant UserAgentAlias => 'Gungho_Engine_POE_UserAgent_Alias';
 
@@ -19,6 +19,7 @@ sub setup
 {
     my $self = shift;
     $self->alias('MainComp');
+    $self->loop_delay( $self->config->{loop_delay} ) if $self->config->{loop_delay};
     $self->next::method(@_);
 }
 
@@ -81,14 +82,24 @@ sub session_stop
 sub session_loop
 {
     my ($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
+    $self->loop_alarm(undef);
+
     my $c = $heap->{CONTEXT};
 
-    if ($c->has_requests) {
-        foreach my $request ( $c->get_requests() ) {
-            $self->send_request($c, $request);
-        }
+    if (! $c->is_running) {
+        $c->log->debug("is_running = 0, waiting for other queued states to finish...\n") if $c->log->is_debug;
+        return;
+    }
 
-        $kernel->yield('session_loop');
+    $c->dispatch_requests();
+
+    my $alarm_id = $self->loop_alarm;
+    if (! $alarm_id) {
+        my $delay = $self->loop_delay;
+        if (! defined $delay || $delay <= 0) {
+            $delay = 5;
+        }
+        $self->loop_alarm($kernel->delay_set('session_loop', $delay));
     }
 }
 
@@ -109,6 +120,14 @@ sub handle_response
     my $req = $req_packet->[0];
     my $res = $res_packet->[0];
     $c->run_hook('engine.handle_response', { request => $req, response => $res });
+
+    # Do we support auth challenge ?
+    my $code = $c->can('check_authentication_challenge');
+    if ( $code ) {
+        # return if auth has taken care of the response
+        return if $code->($c, $req, $res);
+    }
+        
     $c->handle_response($req, $res);
 }
 
@@ -125,6 +144,7 @@ Gungho::Engine::POE - POE Engine For Gungho
   engine:
     module: POE
     config:
+      loop_delay: 0.5
       client:
         agent:
           - AgentName1
