@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Provider/Simple.pm 6402 2007-04-06T17:34:44.214589Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Provider/Simple.pm 6471 2007-04-11T23:53:04.413297Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -7,6 +7,7 @@ package Gungho::Provider::Simple;
 use strict;
 use warnings;
 use base qw(Gungho::Provider);
+use Gungho::Request;
 
 __PACKAGE__->mk_accessors($_) for qw(requests);
 
@@ -18,6 +19,23 @@ sub new
     $self;
 }
 
+sub setup
+{
+    my $self = shift;
+
+    my $url = $self->config->{url};
+    if ($url && ! ref($url) ) {
+        $url = [ $url ];
+    }
+
+    foreach my $u (@$url) {
+        $self->add_request(
+            Gungho::Request->new(GET => $u)
+        );
+    }
+    $self->next::method(@_);
+}
+
 sub add_request
 {
     my ($self, $req) = @_;
@@ -27,14 +45,27 @@ sub add_request
     $self->has_requests(1);
 }
 
-sub get_requests
+sub pushback_request
 {
-    my $self = shift;
+    my ($self, $c, $request) = @_;
+    $c->is_running(1);
+    $self->add_request($request);
+}
 
-    my $list = $self->requests;
+sub dispatch
+{
+    my ($self, $c) = @_;
+
+    my $requests = $self->requests;
     $self->requests([]);
-    $self->has_requests(0);
-    return @$list;
+    while (@$requests) {
+        $self->dispatch_request($c, shift @$requests);
+    }
+
+    if (scalar @{ $self->requests } <= 0) {
+        $self->has_requests(0);
+        $c->is_running(0);
+    }
 }
 
 1;
@@ -57,15 +88,18 @@ Gungho::Provider::Simple - An In-Memory, Simple Provider
 
 Creates a new instance.
 
+=head2 setup($c)
+
+Sets up the provider.
+
 =head2 add_request($request)
 
 Adds a new request to the provider.
 
-=head2 get_requests()
-
-Returns the list of requests in the provider. The list is set to an empty
-list after the call, and has_requests is set to 0
+=head2 pushback_request($c, $request)
 
+=head2 dispatch()
 
+dispatches the requests
 
 =cut
\ No newline at end of file
